use crate::entity::prelude::Account;
use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*, types::ParseMode};

#[derive(Clone, Copy, EnumIter, DeriveColumn, Debug)]
enum Query {
    AccountName,
}

pub async fn get_accounts(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
) -> crate::Result<()> {
    let user_id = msg.from().unwrap().id.0;
    let mut account_names = Account::get_names(user_id, &db).await?.into_iter();
    let mut result = match account_names.next() {
        Some(name) => format!("Accounts:\n`{name}`"),
        None => {
            bot.send_message(msg.chat.id, "No accounts found").await?;
            return Ok(());
        }
    };
    for name in account_names {
        result.reserve(name.len() + 3);
        result.push_str("\n`");
        result.push_str(&name);
        result.push('\'')
    }
    bot.send_message(msg.chat.id, result)
        .parse_mode(ParseMode::MarkdownV2)
        .await?;
    Ok(())
}
