use sea_orm::prelude::*;
use teloxide::{
    adaptors::{throttle::Limits, Throttle},
    filter_command,
    prelude::*,
    utils::command::BotCommands,
};

mod add_account;
mod default;
mod get_account;
mod get_accounts;
mod help;
mod set_master_pass;

#[derive(BotCommands, Clone)]
#[command(rename_rule = "snake_case")]
enum Command {
    #[command()]
    Help,
    #[command(parse_with = "split")]
    AddAccount(String, String, String, String),
    #[command(parse_with = "split")]
    GetAccount(String, String),
    #[command()]
    GetAccounts,
    #[command()]
    SetMasterPass(String),
}

pub fn get_dispatcher(
    token: String,
    db: DatabaseConnection,
) -> Dispatcher<Throttle<Bot>, crate::Error, teloxide::dispatching::DefaultKey> {
    let bot = Bot::new(token).throttle(Limits::default());
    Dispatcher::builder(
        bot,
        Update::filter_message()
            .branch(
                filter_command::<Command, _>()
                    .branch(dptree::case![Command::Help].endpoint(help::help))
                    .branch(
                        dptree::case![Command::AddAccount(name, login, password, master_pass)]
                            .endpoint(add_account::add_account),
                    )
                    .branch(
                        dptree::case![Command::GetAccount(name, master_pass)]
                            .endpoint(get_account::get_account),
                    )
                    .branch(
                        dptree::case![Command::GetAccounts].endpoint(get_accounts::get_accounts),
                    )
                    .branch(
                        dptree::case![Command::SetMasterPass(param)]
                            .endpoint(set_master_pass::set_master_pass),
                    ),
            )
            .branch(dptree::endpoint(default::default)),
    )
    .dependencies(dptree::deps![db])
    .enable_ctrlc_handler()
    .build()
}
